"use strict";
/**
 * Text - Rich Text
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RichText = void 0;
const getLink_1 = require("../../utils/getLink/getLink");
const getExcerpt_1 = require("../../utils/getExcerpt/getExcerpt");
const filters_1 = require("../../utils/filters/filters");
const isString_1 = require("../../utils/isString/isString");
const isArray_1 = require("../../utils/isArray/isArray");
const isObject_1 = require("../../utils/isObject/isObject");
const isHeading_1 = require("../../utils/isHeading/isHeading");
/**
 * Function - check if string contains shortcode
 *
 * @private
 * @param {string} tag
 * @param {string} content
 * @return {boolean}
 */
const _containsShortcode = (tag = '', content = '') => {
    if (tag === 'p' && content.charAt(0) === '[' && content.charAt(content.length - 1) === ']') {
        return true;
    }
    return false;
};
/**
 * Function - recursively output content
 *
 * @private
 * @param {import('./RichTextTypes').RichTextContentProps} args
 * @return {Promise<string>}
 */
const _getContent = async (args) => {
    const { content = [], props, dataAttr = false } = args;
    let { _output = '' } = args;
    for (let i = 0; i < content.length; i += 1) {
        const item = content[i];
        const { link = '', internalLink, content: c } = item;
        let { tag = '' } = item;
        let cc = c;
        /* Nested content */
        if ((0, isArray_1.isArrayStrict)(c)) {
            cc = await _getContent({
                content: c,
                props,
                dataAttr
            });
        }
        /* Attributes */
        const attrs = [];
        if (dataAttr) {
            attrs.push(`data-rich="${tag}"`);
        }
        /* Link */
        if (tag === 'a') {
            let anchorLink = link;
            if (internalLink !== undefined) {
                anchorLink = (0, getLink_1.getLink)(internalLink);
            }
            if (anchorLink !== '') {
                attrs.push(`href="${anchorLink}"`);
            }
        }
        /* Filter output */
        let outputStr = '';
        if ((0, isString_1.isString)(cc)) {
            const richTextContentFilterArgs = {
                args: item,
                props
            };
            const ccc = await (0, filters_1.applyFilters)('richTextContent', cc, richTextContentFilterArgs);
            if ((0, isString_1.isString)(ccc)) {
                cc = ccc;
            }
            outputStr += cc;
        }
        if (_containsShortcode(tag, outputStr)) {
            tag = '';
        }
        /* Output */
        if ((0, isString_1.isStringStrict)(tag) && outputStr.trim() !== '') {
            outputStr = `<${tag}${(attrs.length > 0) ? ` ${attrs.join(' ')}` : ''}>${outputStr}</${tag}>`;
        }
        const richTextContentOutput = {
            args: item,
            props
        };
        outputStr = await (0, filters_1.applyFilters)('richTextContentOutput', outputStr, richTextContentOutput);
        _output += outputStr;
    }
    /* Output */
    return _output;
};
/**
 * Function - output rich text
 *
 * @param {import('./RichTextTypes').RichTextProps} props
 * @return {Promise<string>}
 */
const RichText = async (props) => {
    /* Props must be object */
    if (!(0, isObject_1.isObjectStrict)(props)) {
        return '';
    }
    props = await (0, filters_1.applyFilters)('richTextProps', props, { renderType: 'richText' });
    /* Filtered props must be object */
    if (!(0, isObject_1.isObjectStrict)(props)) {
        return '';
    }
    let { args } = props;
    args = (0, isObject_1.isObjectStrict)(args) ? args : {};
    const { content = [], classes = '', link = '', internalLink, textStyle = '', headingStyle = '', align = '', style = '', attr = '', dataAttr = true } = args;
    let { tag = '' } = args;
    /* Hr */
    if (tag === 'hr') {
        return '<hr>';
    }
    /* Check if heading */
    const isSectionHeading = (0, isHeading_1.isHeading)(tag);
    /* Classes */
    const classesArr = [];
    if ((0, isString_1.isStringStrict)(classes)) {
        classesArr.push(classes);
    }
    if ((0, isString_1.isStringStrict)(textStyle)) {
        classesArr.push(textStyle);
    }
    if ((0, isString_1.isStringStrict)(headingStyle) && isSectionHeading) {
        classesArr.push(headingStyle);
    }
    if ((0, isString_1.isStringStrict)(align)) {
        classesArr.push(align);
    }
    /* Generate output */
    let output = '';
    let headingStr = '';
    let headingObj;
    if ((0, isString_1.isStringStrict)(content)) {
        output = content;
        headingStr = content;
    }
    if ((0, isArray_1.isArrayStrict)(content)) {
        headingObj = content;
        output = await _getContent({
            content,
            props,
            dataAttr
        });
    }
    /* Attributes */
    const attrs = [];
    if (dataAttr) {
        attrs.push(`data-rich="${tag}"`);
    }
    if (isSectionHeading) {
        const headingContents = (0, getExcerpt_1.getExcerpt)({
            limit: 10,
            limitExcerpt: true,
            excerpt: headingStr,
            content: headingObj
        });
        const id = headingContents
            .trim()
            .replace('&hellip;', '')
            .replace(/[^\w\s]|_/g, '')
            .replace(/\s+/g, '-')
            .toLowerCase();
        if (headingContents !== '' && id !== '') {
            attrs.push(`id=${id}`);
            if ((0, isArray_1.isArray)(props.headings)) {
                props.headings.push({
                    id,
                    title: headingContents,
                    type: tag
                });
            }
        }
    }
    if (tag === 'a') {
        let anchorLink = link;
        const inLink = (0, getLink_1.getLink)(internalLink);
        if ((0, isString_1.isStringStrict)(inLink)) {
            anchorLink = inLink;
        }
        if (anchorLink !== '') {
            attrs.push(`href="${anchorLink}"`);
        }
    }
    if (classesArr.length > 0) {
        attrs.push(`class="${classesArr.join(' ')}"`);
    }
    if ((0, isString_1.isStringStrict)(style)) {
        attrs.push(`style="${style}"`);
    }
    if ((0, isString_1.isStringStrict)(attr)) {
        attrs.push(attr);
    }
    /* Output */
    if (_containsShortcode(tag, output)) {
        tag = '';
    }
    if (tag !== '' && output.trim() !== '') {
        output = `<${tag}${(attrs.length > 0) ? ` ${attrs.join(' ')}` : ''}>${output}</${tag}>`;
    }
    output = await (0, filters_1.applyFilters)('richTextOutput', output, props);
    return output;
};
exports.RichText = RichText;
